import {clearAllStorage, getToken} from "@/storage";
import {PAGES} from "@/config";
import {navigateTo,reLaunch} from "@/utils/uniNavigate";

//每次请求的requestTask都会存入这里面
const requestStore = {
    debug: false,
    state: {},
    add(key, value) {
        this.state[key] = value
        if (this.debug) {
            console.log(`requestStore add ${key}`);
            console.log('this.state', JSON.parse(JSON.stringify(this.state)));
        }
    },
    del(key) {
        delete this.state[key]
        if (this.debug) {
            console.log(`requestStore del ${key}`);
            console.log('this.state', JSON.parse(JSON.stringify(this.state)));
        }
    },
    update(key, value) {
        this.state[key] = value
        if (this.debug) {
            console.log(`requestStore update ${key}`);
            console.log('this.state', JSON.parse(JSON.stringify(this.state)));

        }
    }
}

/*

* */
function request({
                     url,
                     method,
                     data,
                     params,
                     noNeedLogin = false, //是否要登陆才能查询否则会被定向到登录页面 | boolean | 默认值false,代表需要有token才可调用接口
                     cancelDuplicateRequest = true //完全相同连续两次请求是否取消上次未完成的请求 | boolean | 默认true，代表会自动取消上次未完成的相同请求
                 }) {

    //token
    const Authorization = getToken()
    //请求头
    let header = {
        "content-type": "",
        'Authorization': Authorization
    }
    if (method !== 'GET') {
        if (data) header["content-type"] = 'application/json'
        if (params) header["content-type"] = 'application/x-www-form-urlencoded'
    }

    if (!data)
        data = params


    /**
     * 是否可以取调用接口
     * @returns boolean
     */
    const check = () => {
        //没有token ，且需要登录
        if (!Authorization && !noNeedLogin) {
            uni.showToast({
                title: "登录认证为空，请重新登录",
                icon: "none",
                duration: 1500,
            })
            clearAllStorage()
            reLaunch(PAGES.login)
            return false
        } else
            return true
    }
    return new Promise((resolve, reject) => {
        if (check()) {
            const requestStore_key = url + JSON.stringify(data) + method
            const newRequest = uni.request({
                url:  process.env.VUE_APP_API_BASE_URL + url,
                method: method,
                data: data,
                header,
                success: function (res) {
                    //正常
                    if (res.statusCode === 200) {
                        /*这里可以跟据自己服务端返回的结构类型进行修改*/
                        const {code, message, result} = res.data
                        if (code === 0)
                            resolve(result)
                        else if (code === 401) {
                            uni.showToast({
                                title:'登录已经过期，请重新登录',
                                icon:'none',
                                success:()=>{
                                    reLaunch(PAGES.login)
                                }
                            })
                        }
                        else if (code === -1) {
                            reject(message)
                        }
                        else reject('请求出错啦 code:' + code)
                    }
                    //其他错误
                    else {
                        const message = '请求出错啦 code:' + res.statusCode
                        reject(message)
                    }
                },
                fail: function (error) {
                    //如果不是因为主动取消请求而报的错，则reject错误，否则什么都不做
                    if (error.errMsg !== "request:fail abort") {
                        reject('请求出错啦');
                    }
                },
                complete: function () {
                    //删除store中标识的这个请求
                    requestStore.del(requestStore_key)
                }
            });

            const oldRequest = requestStore.state[requestStore_key]
            //如果已经有该请求
            if (oldRequest) {
                //中断此次请求 (如果cancelDuplicateRequest为true的话)
                if (cancelDuplicateRequest)
                    oldRequest.abort()
                //更新为当前的请求
                requestStore.update(requestStore_key, newRequest)
            }
            //还没有这个请求
            else {
                //添加此次请求
                requestStore.add(requestStore_key, newRequest)
            }
        } else
            reject('Authorization为空')
    });

}


function get(e) {
    return request({
        ...e,
        method: 'GET'
    });
}

function post(e) {
    return request({
        ...e,
        method: 'POST'
    });
}

function put(e) {
    return request({
        ...e,
        method: 'PUT'
    });
}

function deleting(e) {
    return request({
        ...e,
        method: 'DELETE'
    });
}


export {
    get,
    post,
    deleting,
    put,
}
